<?php
/*

Copyright (c)2002-2003 DuckCorp(tm) and RtpNet(tm)



This file is part of LMS.

LMS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

LMS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LMS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



/**
* LMS Unix Group Manager CLass
*
* @access	public
*/
class LmsUnixGroupManager extends LmsObjectManager
{

	/**
	* Constructor
	*
	* @access	public
	* @param	object DftkLdapConnection	&$ldap_connection
	*/
	function LmsUnixGroupManager(&$tracemgr, &$ldap_connection)
	{
		global		$config;

		LmsObjectManager::LmsObjectManager(&$tracemgr, &$ldap_connection);

		$this->_dn_attr = "cn";
		$this->_root_dn = "ou=UnixGroups,ou=Groups,".$config[base];
		$this->_propagation_key_list = array('gidNumber');
		$this->_check_key_list = array();
		$this->_base_objectclasses = array('top', 'posixGroup', 'lmsUnixGroup');

		$this->_register_attr("Host Name", "cn", LMS_ATYPE_STR, 30, 1, true);
		$this->_register_attr("GID Number", "gidnumber", LMS_ATYPE_INT, null, 1, true);
		$this->_register_attr("Password", "userpassword", LMS_ATYPE_STR, 100, 1, false);
		$this->_register_attr("Secondary Group Members", "memberuid", LMS_ATYPE_STR, 15, 100, false);
		$this->_register_attr("Description", "description", LMS_ATYPE_STR, 300, 1, false);
		
		register_shutdown_function(array(&$this, "_LmsUnixGroupManager"));
	}

	/**
	* Destructor
	*
	* @access	public
	*/
	function _LmsUnixGroupManager()
	{
	}

	/**
	* Find Group by GID
	*
	* @access	public
	* @param	string			$gid			GID of the Group
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &find_by_gid($gid)
	{
		$r =& $this->_lc->search($this->_root_dn, "cn=".$gid, $this->_ent);
		if ($r->has_error)
			$r->add_event('lms-api_ugroupfindgse', $gid);
		else
		{
			$nb = $r->get_result('count');

			if ($nb!=0)
			{
				$this->_entry_to_attrs(0, &$r);
				$r->set_result('count', $this->_ent_get_count());
			}
			else
				$r->add_event('lms-api_ugroupfindgnf', $gid);

			if ($n>0)
				$r->add_event('lms-api_probconsit');
		}

		return $r;
	}

	/**
	* Find Group by GID Number
	*
	* @access	public
	* @param	integer			$gidnumber		GID Number of the Group
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &find_by_gidnumber($gidnumber)
	{
		$r =& $this->_lc->search($this->_root_dn, "gidNumber=".$gidnumber, $this->_ent);
		if ($r->has_error)
			$r->add_event('lms-api_ugroupfindgnse', $gidnumber);
		else
		{
			$nb = $r->get_result('count');

			if ($nb!=0)
			{
				$this->_entry_to_attrs(0, &$r);
				$r->set_result('count', $this->_ent_get_count());
			}
			else
				$r->add_event('lms-api_ugroupfindgnnf', $gidnumber);

			if ($n>0)
				$r->add_event('lms-api_probconsit');
		}

		return $r;
	}

	/**
	* Give the GID corresponding to a GID Number
	*
	* @access	public
	* @param	integer			$gidnumber		GID Number of the User
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &get_gid_from_gidnumber($gidnumber)
	{
		$r =& $this->_lc->search($this->_root_dn, "gidNumber=".$gidnumber, $this->_ent, array('cn'));
		if ($r->has_error)
			$r->add_event('lms-api_ugroupconvgngse', $gidnumber);
		else
		{
			$nb = $r->get_result('count');

			if ($nb!=0)
			{
				$this->_entry_to_attrs(0);
				$r->set_result('cn', $this->_attrs['cn']['value']);
				$r->set_result('gid', $this->_attrs['cn']['value']);
			}
			else $r->add_event('lms-api_ugroupconvgngnf', $gidnumber);

			if ($n>0)
				$r->add_event('lms-api_probconsit');
		}

		return $r;
	}

	/**
	* Give the GID Number corresponding to a GID
	*
	* @access	public
	* @param	string			$gid			GID Number of the User
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &get_gidnumber_from_gid($gid)
	{
		$r =& $this->_lc->search($this->_root_dn, "cn=".$gid, $this->_ent, array('gidnumber'));
		if ($r->has_error)
			$r->add_event('lms-api_ugroupconvggnse', $gid);
		else
		{
			$nb = $r->get_result('count');

			if ($nb!=0)
			{
				$this->_entry_to_attrs(0);
				$r->set_result('gidnumber', $this->_attrs['gidnumber']['value']);
			}
			else $r->add_event('lms-api_ugroupconvggnnf', $gid);

			if ($n>0)
				$r->add_event('lms-api_probconsit');
		}

		return $r;
	}

	/**
	* Get a Group list
	*
	* Always fetched attributs : objectclass, cn
	*
	* @access	public
	* @param	array			$attr			Complementary attributs to fetch
	* @param	string			$crit			Alternative criterias
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &get_list($attr = array(), $crit = "")
	{
		if (!is_array($attr))
			$attr = array();
		$attr = array_merge(array('objectclass', 'cn', 'gidnumber'), $attr);
		if (!$crit)
			$crit = "cn=*";

		$r =& $this->_lc->search($this->_root_dn, $crit, $this->_ent, $attr, false);

		if ($r->has_error())
			 $r->add_event('lms-api_ugrouplistse', $crit);
		else
		{
			$this->_entry_to_attrs(0, &$r);
			$r->set_result('count', $this->_ent_get_count());
		}

		return $r;
	}

	/**
	* Give the first unused GID Number
	*
	* @access	public
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &get_first_free_gidnumber()
	{
		$r =& $this->_lc->search($this->_root_dn, "cn=*", $this->_ent, array('gidnumber'), false);

		if ($r->has_error())
			$r->add_event('lms-api_ugroupfreegnse');
		else
		{
			$z = array();
			$ret = 0;
			for($i=0; $i<$this->_ent_get_count(); $i++)
			{
				$this->_entry_to_attrs($i);
				$z[$i] = $this->_attrs['gidnumber']['value'];
			}
			for ($i=5000; $i<=65535; $i++)
			{
				if (!in_array($i, $z))
				{
					$ret = $i;
					break;
				}
			}
			
			if ($ret!=0)
				$r->set_result('first_free_gidnumber', $ret);
			else
				$r->add_event('lms-api_ugroupfreegnnf');
		}
		
		return $r;
	}

	/**
	* Do what need to be done before applying changes (OVERLOAD)
	*
	* @access	private
	*/
	function _pre_commit_changes()
	{
		LmsObjectManager::_pre_commit_changes();

		if ($this->_attrs['userpassword']['changed'])
		{
			if (substr($this->_attrs['userpassword']['value'], 0, 7)!="{crypt}")
			{
				$this->_set_attr_value('userpassword', "{crypt}".dftk_unix_crypt($this->_attrs['userpassword']['value']));
			}
		}
	}

}

?>
